@@ -1,7 +1,7 @@
 package App::shcompgen;
 
-our $DATE = '2016-10-28'; # DATE
-our $VERSION = '0.28'; # VERSION
+our $DATE = '2016-11-03'; # DATE
+our $VERSION = '0.29'; # VERSION
 
 use 5.010001;
 use strict;
@@ -83,8 +83,7 @@ _
     bash_global_dir => {
         summary => 'Directory to put completions scripts',
         schema  => ['array*', of => 'str*'],
-        default => ['/usr/share/bash-completion/completions',
-                    '/etc/bash_completion.d'],
+        default => ['/etc/bash/completions'],
         tags => ['common'],
     },
     bash_per_user_dir => {
@@ -96,7 +95,7 @@ _
     fish_global_dir => {
         summary => 'Directory to put completions scripts',
         schema  => ['array*', of => 'str*'],
-        default => ['/usr/share/fish/completions', '/etc/fish/completions'],
+        default => ['/etc/fish/completions'],
         tags => ['common'],
     },
     fish_per_user_dir => {
@@ -189,11 +188,9 @@ sub _set_args_defaults {
 
     $args->{global} //= ($> ? 0:1);
 
-    $args->{bash_global_dir}   //= ['/usr/share/bash-completion/completions',
-                                    '/etc/bash_completion.d'];
+    $args->{bash_global_dir}   //= ['/etc/bash/completions'];
     $args->{bash_per_user_dir} //= ["$ENV{HOME}/.config/bash/completions"];
-    $args->{fish_global_dir}   //= ['/usr/share/fish/completions',
-                                    '/etc/fish/completions'];
+    $args->{fish_global_dir}   //= ['/etc/fish/completions'];
     $args->{fish_per_user_dir} //= ["$ENV{HOME}/.config/fish/completions"];
     $args->{tcsh_global_dir}   //= ['/etc/tcsh/completions'];
     $args->{tcsh_per_user_dir} //= ["$ENV{HOME}/.config/tcsh/completions"];
@@ -287,6 +284,42 @@ sub _gen_completion_script {
         };
     }
 
+    if (($detres->[3]{'func.completer_type'} // '') =~ /\AGetopt::Std\z/) {
+        require Data::Dmp;
+        require Getopt::Long::Util;
+        require Getopt::Std::Dump;
+
+        my $content;
+        my $dump_res = Getopt::Std::Dump::dump_getopt_std_script(
+            filename => $progpath,
+            skip_detect => 1,
+        );
+        if ($dump_res->[0] != 200) {
+            $log->errorf("Can't dump Getopt::Std script '%s': %s", $progpath, $dump_res);
+            $script = "# Can't dump Getopt::Std script '$progpath': $dump_res->[0] - $dump_res->[1]\n";
+            goto L1;
+        }
+        $content = join(
+            "",
+            "#!$^X\n",
+            "use Getopt::Long::Complete;\n",
+            "my \$spec = ", Data::Dmp::dmp(
+                Getopt::Long::Util::gen_getopt_long_spec_from_getopt_std_spec(
+                    is_getopt => $dump_res->[2][0] eq 'getopt' ? 1:0,
+                    spec => $dump_res->[2][1])
+              ), ";\n",
+            "GetOptions(%\$spec);\n",
+        );
+        $comp = ($args{global} ?
+                     $args{helper_global_dir} : $args{helper_per_user_dir}) .
+            "/$prog";
+        $qcomp = String::ShellQuote::shell_quote($comp);
+        push @helper_scripts, {
+            path => $comp,
+            content => $content,
+        };
+    }
+
     if ($shell eq 'bash') {
         if (defined $args) {
 
@@ -580,8 +613,11 @@ sub _detect_prog {
 
             for my $line (@lines) {
                 if ($line =~ /^\s*((?:use|require)\s+
-                                  (Getopt::Long(?:::Complete|::Subcommand|::Descriptive)?|
-                                      Perinci::CmdLine(?:::Any|::Lite|::Classic)))\b/x) {
+                                  (
+                                      Getopt::Std|
+                                      Getopt::Long(?:::Complete|::Subcommand|::Descriptive)?|
+                                      Perinci::CmdLine(?:::Any|::Lite|::Classic)
+                              ))\b/x) {
                     return [200, "OK", 1, {
                         "func.completer_command"=> $prog, # later will be set
                         "func.completer_type"=> $2,
@@ -850,13 +886,12 @@ _shcompgen_loader()
 
     # XXX should we use --bash-{global,per-user}-dir supplied by user here? probably.
     local dirs
+    dirs=(~/.config/bash/completions /etc/bash/completions)
     if [[ "$bc_active" = 1 ]]; then
-        dirs=(~/.config/bash/completions /etc/bash_completion.d /usr/share/bash-completion/completions)
-    else
-        # we don't use bash-completion scripts when bash-completion is not
+        # we only search in bash-completion dirs when bash-completion has been
         # initialized because some of the completion scripts require that
-        # bash-completion system is initialized first
-        dirs=(~/.config/bash/completions)
+        # bash-completion system is initialized first (e.g. _init_completion)
+        dirs+=(/etc/bash_completion.d /usr/share/bash-completion/completions)
     fi
 
     local d
@@ -1107,7 +1142,7 @@ App::shcompgen - Generate shell completion scripts
 
 =head1 VERSION
 
-This document describes version 0.28 of App::shcompgen (from Perl distribution App-shcompgen), released on 2016-10-28.
+This document describes version 0.29 of App::shcompgen (from Perl distribution App-shcompgen), released on 2016-11-03.
 
 =head1 FUNCTIONS
 
@@ -1152,7 +1187,7 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<bash_global_dir> => I<array[str]> (default: ["/usr/share/bash-completion/completions","/etc/bash_completion.d"])
+=item * B<bash_global_dir> => I<array[str]> (default: ["/etc/bash/completions"])
 
 Directory to put completions scripts.
 
@@ -1160,7 +1195,7 @@ Directory to put completions scripts.
 
 Directory to put completions scripts.
 
-=item * B<fish_global_dir> => I<array[str]> (default: ["/usr/share/fish/completions","/etc/fish/completions"])
+=item * B<fish_global_dir> => I<array[str]> (default: ["/etc/fish/completions"])
 
 Directory to put completions scripts.
 
@@ -1291,7 +1326,7 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<bash_global_dir> => I<array[str]> (default: ["/usr/share/bash-completion/completions","/etc/bash_completion.d"])
+=item * B<bash_global_dir> => I<array[str]> (default: ["/etc/bash/completions"])
 
 Directory to put completions scripts.
 
@@ -1299,7 +1334,7 @@ Directory to put completions scripts.
 
 Directory to put completions scripts.
 
-=item * B<fish_global_dir> => I<array[str]> (default: ["/usr/share/fish/completions","/etc/fish/completions"])
+=item * B<fish_global_dir> => I<array[str]> (default: ["/etc/fish/completions"])
 
 Directory to put completions scripts.
 
@@ -1382,7 +1417,7 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<bash_global_dir> => I<array[str]> (default: ["/usr/share/bash-completion/completions","/etc/bash_completion.d"])
+=item * B<bash_global_dir> => I<array[str]> (default: ["/etc/bash/completions"])
 
 Directory to put completions scripts.
 
@@ -1392,7 +1427,7 @@ Directory to put completions scripts.
 
 =item * B<detail> => I<bool>
 
-=item * B<fish_global_dir> => I<array[str]> (default: ["/usr/share/fish/completions","/etc/fish/completions"])
+=item * B<fish_global_dir> => I<array[str]> (default: ["/etc/fish/completions"])
 
 Directory to put completions scripts.
 
@@ -1475,7 +1510,7 @@ Arguments ('*' denotes required arguments):
 
 =over 4
 
-=item * B<bash_global_dir> => I<array[str]> (default: ["/usr/share/bash-completion/completions","/etc/bash_completion.d"])
+=item * B<bash_global_dir> => I<array[str]> (default: ["/etc/bash/completions"])
 
 Directory to put completions scripts.
 
@@ -1483,7 +1518,7 @@ Directory to put completions scripts.
 
 Directory to put completions scripts.
 
-=item * B<fish_global_dir> => I<array[str]> (default: ["/usr/share/fish/completions","/etc/fish/completions"])
+=item * B<fish_global_dir> => I<array[str]> (default: ["/etc/fish/completions"])
 
 Directory to put completions scripts.
 
